#############################################################################################
##
## From Fang Yu(UNMC), Ming-Hui Chen(UCONN) and Lynn Kuo(UCONN)
## This is a trial version of R code for idnetifying DE genes from microarray data
## Allows two or multiple groups under comparison
## Avaiable from May 14 2015
##
## Input: 
## 	Datause:   The microarray dataset, which is organized with each row representing a gene 
##	    	   and each column representing a replicate.
##		   We assume that "datause" follows normal ditribution
## 	nburnin:   The iteration for burn-in
## 	nmcmc:     The nubmer of interation used to obtain posterior estiamtion after burnin
## 	design:    A categorical vector with 1,2 to indicate the group under comaprison
##	    	   For example, an experiment with three replicates per group out of two groups 
##		   will have design=c(1,1,1,2,2,2)
##	prior_sig: Variance parameter in the normal Prior on the mean or difference of the
##		   gene expression across gorups 	
##	prior_h:   Inverse-Gamma prior parameters for the variance of the gene expression values
##	pcut:      The targeted false discovery rate level 	
##	
## Output:
##	order:	   The ordering the genes in a descending support of differential gene expression
##	post_mean: The posterior esimator on the mean expressions
##	post_sig:   The posterior estiamtor on the variance expressions.
##	gammau:	   The calculated gamma value 
##	nDE:	   Total number of idnetified DE genes 
##	DE_sign:   Indicator for identified DE genes (1: DE gene; 0: EE gene)
##
#############################################################################################

CDC.micro3=function(datause,nburnin,nmcmc,design,prior_sig,prior_h,pcut)
{
dyn.load("gamcalr.dll")
dyn.load("fdrcb.dll")


	ngene=nrow(datause)
	mj=as.integer(length(unique(design)))
	ncomp=mj-1

	nrept=ncol(datause)
	nrepv=NULL
	for(i in 1:mj)
	{
		nrepv=c(nrepv,sum(design==i))
	}

	if(! is.double(datause)) {
       print("The read-in data is in wrong data format!") 
       return()
       }

	ttauv=rep(prior_sig,ncomp)
	tlamvec=ttauv
	aar1=prior_h[1]
	bbr1=prior_h[2]
	aapr=prior_h[1]
	bbpr=prior_h[2]

	uu=.Fortran("gamcal",iseed=as.integer(1234),nins=as.integer(1),
	ngene=as.integer(ngene),mj=as.integer(mj),
	nrepv=as.integer(nrepv),nrept=as.integer(nrept),
	rdata=datause,ngen=as.integer(nburnin),ngen1=as.integer(nmcmc),
	aar1=as.double(aar1),bbr1=as.double(bbr1),aapr=as.double(aapr),bbpr=as.double(bbpr),
	ttauv=as.double(ttauv),tlamvec=as.double(tlamvec),
	gammause=matrix(as.double(matrix(0.0,ngene,ncomp)),ngene,ncomp),
	vh0mu=as.double(matrix(0.0,ngene,1)),
	vh0sig=as.double(matrix(0.0,ngene,1)),
	vh1muv=matrix(as.double(matrix(0.0,ngene,mj)),ngene,mj),
	vh1sigv=matrix(as.double(matrix(0.0,ngene,mj)),ngene,mj))


	DE_Y=matrix(0,ngene,ncomp)
	idorder=DE_Y
	nselect=rep(0,ncomp)

	for(nfocus in 1:2)
	{
	nstep=1
	gammause=uu$gammause
	ncompt=2**(2*ncomp)
	fdrresult=.Fortran("fdrcal",iseed=as.integer(1234),pcut=as.double(pcut),nfocus=as.integer(nfocus),
	ngene=as.integer(ngene),
	mj=as.integer(mj),nrept=as.integer(nrept),nrepv=as.integer(nrepv),ngee=as.integer(ngene*0.1),
	aar1=as.double(aar1),bbr1=as.double(bbr1),aapr=as.double(aapr),bbpr=as.double(bbpr),
	ttauv=as.double(ttauv),tlamvec=as.double(tlamvec),
	gammause=gammause,vh1muv=uu$vh1muv,vh1sigv=uu$vh1sigv,vh0mu=uu$vh0mu,
	vh0sig=uu$vh0sig,ngen=as.integer(100),ngen1=as.integer(100),
	nposfindn=as.integer(1) ,ncompt=as.integer(ncompt),
	jju=as.integer(1),fdrf=as.double(0.0),nstep=as.integer(nstep))

	nstep=0
	jju=as.integer(fdrresult$nposfindn/20+1)*20
	fdrresult=.Fortran("fdrcal",iseed=as.integer(1234),pcut=as.double(pcut),nfocus=as.integer(nfocus),
	ngene=as.integer(ngene),
	mj=as.integer(mj),nrept=as.integer(nrept),nrepv=as.integer(nrepv),ngee=as.integer(ngene*0.1),
	aar1=as.double(aar1),bbr1=as.double(bbr1),aapr=as.double(aapr),bbpr=as.double(bbpr),
	ttauv=as.double(ttauv),tlamvec=as.double(tlamvec),
	gammause=gammause,vh1muv=uu$vh1muv,vh1sigv=uu$vh1sigv,vh0mu=uu$vh0mu,
	vh0sig=uu$vh0sig,ngen=as.integer(min(1000,nburnin)),ngen1=as.integer(nmcmc),
	nposfindn=as.integer(1),ncompt=as.integer(ncompt),
	jju=as.integer(jju),fdrf=as.double(0.0),nstep=as.integer(nstep))

	nselect[nfocus]=fdrresult$nposfindn
	idorder[,nfocus]=order(gammause[,nfocus],decreasing=T)
	DE_Y[idorder[1:nselect[nfocus],nfocus],nfocus]=1
	}
	
list("order"=idorder,"post_mean"=uu$vh1muv,"post_sig"=uu$vh1sigv,"gammau"=gammause,"nDE"=nselect,"DE_sign"=DE_Y)
}
